@@ -1,6 +1,7 @@
 // download_install_manager.cc
 //
 //   Copyright (C) 2005-2011 Daniel Burrows
+//   Copyright (C) 2015-2016 Manuel A. Fernandez Montecelo
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -14,8 +15,8 @@
 //
 //   You should have received a copy of the GNU General Public License
 //   along with this program; see the file COPYING.  If not, write to
-//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-//   Boston, MA 02111-1307, USA.
+//   the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+//   Boston, MA 02110-1301, USA.
 
 #include "download_install_manager.h"
 
@@ -25,6 +26,8 @@
 
 #include <aptitude.h>
 
+#include <generic/apt/apt.h>
+
 #include <apt-pkg/acquire-item.h>
 #include <apt-pkg/dpkgpm.h>
 #include <apt-pkg/error.h>
@@ -37,6 +40,7 @@
 
 using namespace std;
 
+
 download_install_manager::download_install_manager(bool _download_only,
 						   const run_dpkg_in_terminal_func &_run_dpkg_in_terminal)
   : log(NULL), download_only(_download_only), pm(new pkgDPkgPM(*apt_cache_file)),
@@ -61,17 +65,19 @@ bool download_install_manager::prepare(OpProgress &progress,
       return false;
     }
 
-  if(!(*apt_cache_file)->save_selection_list(progress))
+  if(!(*apt_cache_file)->save_selection_list(&progress))
     return false;
 
-  progress.Done();
-
   // Abort here so we don't spew random messages below.
   if(_error->PendingError())
     return false;
 
   fetcher = new pkgAcquire;
-  if(fetcher->Setup(&acqlog, aptcfg->FindDir("Dir::Cache::archives")) == false)
+  fetcher->SetLog(&acqlog);
+  // even if we would like to not have to get locks and have to call
+  // fetcher->Run() if no (remote) fetch is needed (see #766122), it is needed
+  // anyway to work with local repositories (see #816537)
+  if (fetcher->GetLock(aptcfg->FindDir("Dir::Cache::archives")) == false)
     {
       delete fetcher;
       fetcher = NULL;
@@ -91,6 +97,9 @@ bool download_install_manager::prepare(OpProgress &progress,
      _error->PendingError())
     {
       _error->Error(_("Internal error: couldn't generate list of packages to download"));
+      // has to be an Error (not e.g. Notice), otherwise "DumpError" (which
+      // later phases rely on) doesn't include the following message
+      _error->Error(_("Perhaps the package lists are out of date, please try 'aptitude update' (or equivalent) first"));
 
       delete fetcher;
       fetcher = NULL;
@@ -135,10 +144,23 @@ download_manager::result download_install_manager::finish_pre_dpkg(pkgAcquire::R
 	}
     }
 
-  if(failed && !pm->FixMissing())
+  const char* fix_missing_cstr = "APT::Get::Fix-Missing";
+  bool fix_missing = aptcfg->FindB(fix_missing_cstr, false);
+  if (failed)
     {
-      _error->Error(_("Unable to correct for unavailable packages"));
-      return failure;
+      if (fix_missing)
+	{
+	  if (!pm->FixMissing())
+	    {
+	      _error->Error(_("Unable to correct for unavailable packages"));
+	      return failure;
+	    }
+	}
+      else
+	{
+	  _error->Error(_("Unable to fetch some packages; try '-o %s=true' to continue with missing packages"), fix_missing_cstr);
+	  return failure;
+	}
     }
 
   log_changes();
@@ -166,13 +188,18 @@ pkgPackageManager::OrderResult download_install_manager::run_dpkg(int status_fd)
   sigfillset(&allsignals);
 
   pthread_sigmask(SIG_UNBLOCK, &allsignals, &oldsignals);
-  pkgPackageManager::OrderResult pmres = pm->DoInstallPostFork(status_fd);
+  std::unique_ptr<APT::Progress::PackageManager> progress;
+  if (status_fd > 0)
+    progress = std::make_unique<APT::Progress::PackageManagerProgressFd>(status_fd);
+  else
+    progress = std::unique_ptr<APT::Progress::PackageManager> { APT::Progress::PackageManagerProgressFactory() };
+  pkgPackageManager::OrderResult pmres = pm->DoInstallPostFork(progress.get());
 
   switch(pmres)
     {
     case pkgPackageManager::Failed:
       _error->DumpErrors();
-      cerr << _("Failed to perform requested operation on package.  Trying to recover:") << endl;
+      //cout << _("Failed to perform requested operation on package.  Trying to recover:") << endl;
       if(system("DPKG_NO_TSTP=1 dpkg --configure -a") != 0) { /* ignore */ }
       break;
     case pkgPackageManager::Completed:
@@ -227,6 +254,11 @@ void download_install_manager::finish_post_dpkg(pkgPackageManager::OrderResult d
       if(log != NULL)
 	log->Complete();
 
+      // after the fix for '"reinstall" planned action is now preserved (Closes:
+      // #255587, #785641)', we need to clear the state of the package,
+      // otherwise it remains forever wanting to be reinstalled
+      bool reset_reinstall = (dpkg_result == pkgPackageManager::Completed) ? true : false;
+
       // We absolutely need to do this here.  Yes, it slows things
       // down, but without this we get stuff like #429388 due to
       // inconsistencies between aptitude's state file and the real
@@ -234,7 +266,10 @@ void download_install_manager::finish_post_dpkg(pkgPackageManager::OrderResult d
       //
       // This implicitly updates the package state file on disk.
       if(!download_only)
-	apt_load_cache(progress, true);
+	{
+	  bool operation_needs_lock = true;
+	  apt_load_cache(progress, true, operation_needs_lock, nullptr, reset_reinstall);
+	}
 
       if(aptcfg->FindB(PACKAGE "::Forget-New-On-Install", false)
 	 && !download_only)
@@ -242,12 +277,29 @@ void download_install_manager::finish_post_dpkg(pkgPackageManager::OrderResult d
 	  if(apt_cache_file != NULL)
 	    {
 	      (*apt_cache_file)->forget_new(NULL);
-	      (*apt_cache_file)->save_selection_list(*progress);
+	      (*apt_cache_file)->save_selection_list(progress);
 	      post_forget_new_hook();
 	    }
 	}
     }
 
+    if (rval == success && !download_only)
+      {
+	if (aptcfg->FindB(PACKAGE "::Clean-After-Install", false))
+	  {
+	    pre_clean_after_install_hook();
+
+	    bool result_ok = aptitude::apt::clean_cache_dir();
+
+	    if (!result_ok)
+	      {
+		rval = failure;
+	      }
+
+	    post_clean_after_install_hook();
+	  }
+      }
+
   k(rval);
 }
 
